﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using System.Collections.Generic;
using System.IO;

using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.ComponentModel;
using System.Collections.ObjectModel;


namespace CustomerManager
{
    [Table]
    public class Customer : INotifyPropertyChanged, INotifyPropertyChanging
    {
        [Column(IsPrimaryKey = true, IsDbGenerated = true, AutoSync = AutoSync.OnInsert)]
        public int CustomerID { get; set; }

        private string nameValue;

        [Column]
        public string Name
        {
            get
            {
                return nameValue;
            }
            set
            {
                NotifyPropertyChanging("Name");
                nameValue = value;
                NotifyPropertyChanged("Name");
            }
        }

        private string addressValue;
        [Column]
        public string Address
        {
            get
            {
                return addressValue;
            }
            set
            {
                NotifyPropertyChanging("Address");
                addressValue = value;
                NotifyPropertyChanged("Address");
            }
        }

        private EntitySet<Session> sessions = new EntitySet<Session>();

        [Association(Storage = "sessions", ThisKey = "CustomerID", OtherKey = "SessionCustomerID")]
        public EntitySet<Session> Sessions
        {
            get
            {
                return sessions;
            }
            set
            {
                sessions = value;
            }
        }

        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this,
                              new PropertyChangedEventArgs(propertyName));
            }
        }

        private void NotifyPropertyChanging(string propertyName)
        {
            if (PropertyChanging != null)
            {
                PropertyChanging(this,
                              new PropertyChangingEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        public event PropertyChangingEventHandler PropertyChanging;
    }

    [Table]
    public class Session : INotifyPropertyChanged, INotifyPropertyChanging
    {

        [Column(IsPrimaryKey = true, IsDbGenerated = true, AutoSync = AutoSync.OnInsert)]
        public int SessionID;

        private string description;

        [Column]
        public string Description
        {
            get
            {
                return description;
            }
            set
            {
                NotifyPropertyChanging("Description");
                description = value;
                NotifyPropertyChanged("Description");
            }
        }

        private int lengthInMins;

        [Column]
        public int LengthInMins
        {
            get
            {
                return lengthInMins;
            }
            set
            {
                NotifyPropertyChanging("LengthInMins");
                lengthInMins = value;
                NotifyPropertyChanged("LengthInMins");
            }
        }

        [Column]
        public int SessionCustomerID;

        private EntityRef<Customer> sessionCustomer = new EntityRef<Customer>();

        [Association(IsForeignKey = true, Storage = "sessionCustomer", ThisKey = "SessionCustomerID")]
        public Customer SessionCustomer
        {
            get
            {
                return sessionCustomer.Entity;
            }
            set
            {
                NotifyPropertyChanging("SessionCustomer");
                sessionCustomer.Entity = value;
                NotifyPropertyChanged("SessionCustomer");
                if (value != null)
                    SessionCustomerID = value.CustomerID;
            }
        }

        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this,
                              new PropertyChangedEventArgs(propertyName));
            }
        }

        private void NotifyPropertyChanging(string propertyName)
        {
            if (PropertyChanging != null)
            {
                PropertyChanging(this,
                              new PropertyChangingEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        public event PropertyChangingEventHandler PropertyChanging;

    }


    public class CustomerDB : DataContext
    {
        public string Name { get; set; }

        public Table<Customer> CustomerTable;

        public Table<Session> SessionTable;

        public CustomerDB(string connection)
            : base(connection)
        {
        }

        public static void MakeTestDB(string connection)
        {
            string[] firstNames = new string[] { "Rob", "Jim", "Joe", "Nigel", "Sally", "Tim" };
            string[] lastsNames = new string[] { "Smith", "Jones", "Bloggs", "Miles", "Wilkinson", "Brown" };

            using (CustomerDB newDB = new CustomerDB(connection))
            {

                if (newDB.DatabaseExists())
                {
                    newDB.DeleteDatabase();
                }

                newDB.CreateDatabase();

                foreach (string lastName in lastsNames)
                {
                    foreach (string firstname in firstNames)
                    {
                        //Construct some customer details
                        string name = firstname + " " + lastName;
                        string address = name + "'s address";
                        Customer newCustomer = new Customer();
                        newCustomer.Name = name;
                        newCustomer.Address = address;
                        newDB.CustomerTable.InsertOnSubmit(newCustomer);
                        newDB.SubmitChanges();
                    }
                }

                Random sessionRand = new Random(1);

                // Now create some random sessions for our customers

                var customers = from Customer customer
                        in newDB.CustomerTable
                                select customer;

                foreach (Customer c in customers)
                {
                    int noOfSessions = sessionRand.Next(4, 20);
                    for (int i = 0; i < noOfSessions; i++)
                    {
                        int sessionLength = sessionRand.Next(5,120);
                        string sessionDesc = "Session" + i.ToString();
                        Session newSession = new Session();
                        newSession.Description = sessionDesc;
                        newSession.LengthInMins = sessionLength;
                        newDB.SessionTable.InsertOnSubmit(newSession);
                        c.Sessions.Add(newSession);
                    }
                    newDB.SubmitChanges();
                }
            }
        }
    }
}
